load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
;************************************************


;----------------------------------------------------------------------
; This function creates a topo map using a 2' topographic file read in
; from a NetCDF file, and map limits read in from a WRF output file.
;----------------------------------------------------------------------
undef("create_topo_map")
function create_topo_map(wks)
local topo_file, a, elev, res, cmap
begin
  f = addfile("./geo_finemesh.nc","r")

  xlat = f->XLAT_M(0,:,:)
  xlon = f->XLONG_M(0,:,:)

  dims = dimsizes(xlat)
  nlat = dims(0)                                ; assign # lat/lon points
  nlon = dims(1)

  i0 = 0
  i1 = nlon -1
  j0 = 0
  j1 = nlat -1

     hh = f->HGT_M(0,:,:)
     hh@lat2d=xlat
     hh@lon2d=xlon
   delete(hh@description)
   delete(hh@units)


;---Will use this later for coloring the contours
   cmap = read_colormap_file("OceanLakeLandSnow")

;---Set some resources for contouring and mapping
  res                    = True
  res@gsnDraw            = False
  res@gsnFrame           = False
  res@gsnAddCyclic     = False

  
  res@cnLinesOn          = False        ; turn off contour lines
  res@cnLineLabelsOn     = False        ; turn off line labels
  res@cnInfoLabelOn      = False        ; turn off info label

  res@cnFillOn           = True         ; turn on contour fill
  ; res@cnFillMode         = "MeshFill"   ; for faster draw
  res@cnFillPalette      = cmap(2:,:)
 
  res@lbBoxLinesOn       = False        ; turn off labelbar box lines
   res@gsnPaperOrientation   = "portrait"
   res@lbOrientation     = "Horizontal"



;---Pick "nice" contour levels
  mnmxint = nice_mnmxintvl( min(hh), max(hh), 18, False)
  res@cnLevelSelectionMode = "ManualLevels"
  res@cnMinLevelValF       = mnmxint(0)
  res@cnMaxLevelValF       = mnmxint(1)
  res@cnLevelSpacingF      = mnmxint(2)/8.   ; Increase the number of levels 

  res@gsnLeftString         = ""
  res@gsnRightString        = ""
  res@pmTitleZone           = 4              ; move main title down a little

  res1 = res
  res1@mpProjection       = "LambertConformal"
  res1@mpLambertParallel1F = f@TRUELAT1
  res1@mpLambertParallel2F = f@TRUELAT2
  res1@mpLambertMeridianF = f@CEN_LON
  res1@mpLimitMode        = "Corners" ; choose range of map
  res1@mpLeftCornerLatF   = xlat(j0,i0)
  res1@mpLeftCornerLonF   = xlon(j0,i0)
  res1@mpRightCornerLatF  = xlat(j1,i1)
  res1@mpRightCornerLonF  = xlon(j1,i1)
  res1@mpFillOn             = False
  res1@mpOutlineOn          = True
  res1@mpDataBaseVersion    = "MediumRes"
  res1@mpOutlineBoundarySets = "GeophysicalAndUSStates"
  res1@pmTickMarkDisplayMode = "Always"
  ;res1@lbTitleString      = "topography (meters)"    ; add a labelbar title
  ;res1@lbTitlePosition    = "Bottom"
  ;res1@pmLabelBarOrthogonalPosF = 0.0001
  ; res1@gsnMaximize        = True         ; maximize plot in frame

;---Create map and return it.
  plot = gsn_csm_contour_map(wks,hh,res1)

  return(plot)
end

; main code
begin
   
    wks = gsn_open_wks("eps", "domain")

    map    = create_topo_map(wks)

;---Attach rivers from shapefile
;  lnres                  = True
;  lnres@gsLineColor      = "navyblue"
;  lnres@gsFillColor      = "navyblue"
;  lnres@gsLineThicknessF = 1.5
;
;---Setting lat/lon limits helps drawing go faster
;  lnres@minlat           = minlat
;  lnres@maxlat           = maxlat
;  lnres@minlon           = minlon
;  lnres@maxlon           = maxlon

;  dum = gsn_add_shapefile_polylines(wks,map,"rv16my07.shp",lnres)

  draw(map)

  frame(wks)




end
